%SMM estimation.
%Since our weigthing matrix is built only with the data from the real
%world, we do not need to make an interative process. In other words: we
%do not need to set a start value for the weigthing matrix W, such as the
%identity matrix I.
%The following code is a modification of the code from Ruge-Murcia
%(November 2010). I have changed it such that one can use it in combination
%with Dynare.
clear all
format short
dynare Hansen

global mom y c n x W sim T M seed 
%Calculating the moments
%Real Data
load DatenDSGE.mat
ym = log(Y) - hpfilter(log(Y),1600);
cm = log(C) - hpfilter(log(C),1600);
nm = log(L) - hpfilter(log(L),1600);
xm = log(I) - hpfilter(log(I),1600);
%Calculating the moments
data = [cm xm nm];
T = length(ym);
nob = size(data,1); % number of observations 
nvar = size(data,2); % number of variables
for i = 1:nvar
      for j = i:nvar
        if (i==1) && (j==1)%calculation of variance
            vars = data(2:nob,i).*data(2:nob,j);
        else%calculation of covariance
            vars = [vars data(2:nob,i).*data(2:nob,j)];  
        end   
      end
end
autoc1 = data(2:nob,:).*data(1:nob-1,:);%calculation of the autocorrelation of first order
autoc2 = data(3:nob,:).*data(1:nob-2,:);%calculation of the autocorrelation of second order
autoc2 = [ones(1,size(autoc2,2)); autoc2];%add ones to make the moments vectors of the same length
mom = [mean(vars)';mean(autoc1)';mean(autoc2)'];%calculation for the miniziation in the distance.m file
tau = 5;%As Ruge-Murcia (2010,CIREQ) suggests.
M = tau*T; %Length of simulation
seed = floor(rand*10000);%Set the seed for the simulation, as Mark (2003) suggests.
%Calculating the optimal weigthing matrix with the Newey-West method with
%Barlett kernel.
m = floor(4*T^(2/9));%the lag length of the Newey-West method
%The following code is from Ruge-Murcia and follows Mark's persentation in
%chapter 2.3 Simulated Method of Moments.
nob = size(vars,1)+1;
u = (vars-(ones(size(vars,1),1)*mean(vars)))';
u = [u;(autoc1-(ones(size(autoc1,1),1)*mean(autoc1)))'];
u = [u;(autoc2-(ones(size(autoc2,1),1)*mean(autoc2)))'];
w = u*u';
w = w/(nob-1);
for j = 1:m
		sigma = u(:,j+1:nob-1)*u(:,1:nob-1-j)';
		sigma = sigma + sigma';
		sigma = sigma/(nob-1);
		w = w + (1-j/(m+1))*sigma;
end
W = (1+T/M)*w;
W = inv(W);%The optimal weigthing matrix.
startvalue = [betatrans psi rhotrans sigmatrans];%setting the start values for the estimated parameters
options = optimset('Display','iter','MaxFunEvals',15000,'MaxIter',15000);
[erg fval] = fminunc('Distance',startvalue,options);
disp('Estimation results')
betatrans = erg(1);
beta = betatrans^2/(1+betatrans^2)
psi = erg(2)
rhotrans = erg(3);
rho =  rhotrans^2/(1+rhotrans^2)
sigmatrans = erg(4);
sigma = abs(sigmatrans)